@@ -1137,7 +1137,7 @@ QAbstractItemView *MainWindow::constructElementTable<model::GateContainerModel>(
 {
     auto *tree = new QTreeView(parent);
     auto *tableModel = new model::GateContainerModel(guiModel, tree);
-    auto *proxyModel = new model::SortFilterProxyModel(tree);
+    auto *proxyModel = new model::GateSortFilterProxyModel(tree);
     proxyModel->setSourceModel(tableModel);
     tree->setModel(proxyModel);
     tree->setSelectionBehavior(QAbstractItemView::SelectRows);
@@ -1291,7 +1291,7 @@ void MainWindow::resetReportWidget(std::unique_ptr<core::RiskAnalysis> analysis)
         }
     } nameExtractor;
     for (const core::RiskAnalysis::Result &result : m_analysis->results()) {
-        QString name = boost::apply_visitor(nameExtractor, result.id);
+        QString name = boost::apply_visitor(nameExtractor, result.id.target);
         auto *widgetItem = new QTreeWidgetItem({name});
         ui->reportTreeWidget->addTopLevelItem(widgetItem);
 
