@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2016 Olzhas Rakhimov
+ * Copyright (C) 2015-2017 Olzhas Rakhimov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,8 +23,8 @@
 namespace scram {
 namespace core {
 
-Analysis::Analysis(const Settings& settings)
-    : kSettings_(settings),
+Analysis::Analysis(Settings settings)
+    : settings_(std::move(settings)),
       analysis_time_(0) {}
 
 Analysis::~Analysis() = default;  ///< Pure virtual destructor.
