@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2016 Olzhas Rakhimov
+ * Copyright (C) 2015-2017 Olzhas Rakhimov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 #include <boost/noncopyable.hpp>
 #include <boost/smart_ptr/intrusive_ptr.hpp>
 
-#include "boolean_graph.h"
+#include "pdag.h"
 #include "settings.h"
 
 namespace scram {
@@ -49,6 +49,29 @@ using IntrusivePtr = boost::intrusive_ptr<T>;
 template <class T>
 class Vertex;  // Manager of its own entry in the unique table.
 
+/// Provides pointer and reference cast wrappers for intrusive Vertex pointers.
+///
+/// @tparam T  The type of the main functional BDD vertex.
+/// @tparam W  The wrapper type to cast to.
+template <class T, class W = T>
+struct IntrusivePtrCast {
+  /// @param[in] vertex  Pointer to a Vertex known to be convertible to W.
+  ///
+  /// @returns The cast reference-counted pointer to the vertex,
+  ///          or the cast reference (without count increment!) to the vertex.
+  ///
+  /// @pre static_cast<W>(Vertex<T>) is successful.
+  ///
+  /// @{
+  static IntrusivePtr<W> Ptr(const IntrusivePtr<Vertex<T>>& vertex) {
+    return boost::static_pointer_cast<W>(vertex);
+  }
+  static W& Ref(const IntrusivePtr<Vertex<T>>& vertex) {
+    return static_cast<W&>(*vertex);
+  }
+  /// @}
+};
+
 /// A weak pointer to store in BDD unique tables.
 /// This weak pointer is unique pointer as well
 /// because vertices should not be easily shared among multiple BDDs.
@@ -190,7 +213,7 @@ class Vertex : private boost::noncopyable {
 ///
 /// @tparam T  The type of the main functional BDD vertex.
 template <class T>
-class Terminal : public Vertex<T> {
+class Terminal : public Vertex<T>, public IntrusivePtrCast<T, Terminal<T>> {
  public:
   /// @param[in] value  True or False (1 or 0) terminal.
   explicit Terminal(bool value) : Vertex<T>(value) {}
@@ -201,15 +224,6 @@ class Terminal : public Vertex<T> {
   ///       Non-terminal if-then-else vertices should never have
   ///       identifications of value 0 or 1.
   bool value() const { return Vertex<T>::id(); }
-
-  /// Recovers a shared pointer to Terminal from a pointer to Vertex.
-  ///
-  /// @param[in] vertex  Pointer to a Vertex known to be a Terminal.
-  ///
-  /// @return Casted pointer to Terminal.
-  static IntrusivePtr<Terminal<T>> Ptr(const IntrusivePtr<Vertex<T>>& vertex) {
-    return boost::static_pointer_cast<Terminal<T>>(vertex);
-  }
 };
 
 /// Representation of non-terminal vertices in BDD graphs.
@@ -218,7 +232,7 @@ class Terminal : public Vertex<T> {
 ///
 /// @tparam T  The type of the main functional BDD vertex.
 template <class T>
-class NonTerminal : public Vertex<T> {
+class NonTerminal : public Vertex<T>, public IntrusivePtrCast<T> {
   using VertexPtr = IntrusivePtr<Vertex<T>>;  ///< Convenient change point.
 
   /// Default logic for getting signature high and low ids.
@@ -297,15 +311,6 @@ class NonTerminal : public Vertex<T> {
  protected:
   ~NonTerminal() = default;
 
-  /// Cuts off this node from its high and low branches.
-  /// This is for destructive operations on the BDD graph.
-  ///
-  /// @pre These branches are not going to be used again.
-  void CutBranches() {
-    high_.reset();
-    low_.reset();
-  }
-
  private:
   VertexPtr high_;  ///< 1 (True/then) branch in the Shannon decomposition.
   VertexPtr low_;  ///< O (False/else) branch in the Shannon decomposition.
@@ -361,15 +366,6 @@ class Ite : public NonTerminal<Ite> {
   /// @param[in] value  Calculation results for importance factor.
   void factor(double value) { factor_ = value; }
 
-  /// Recovers a shared pointer to Ite from a pointer to Vertex.
-  ///
-  /// @param[in] vertex  Pointer to a Vertex known to be an Ite.
-  ///
-  /// @return Casted pointer to Ite.
-  static IntrusivePtr<Ite> Ptr(const IntrusivePtr<Vertex<Ite>>& vertex) {
-    return boost::static_pointer_cast<Ite>(vertex);
-  }
-
  private:
   bool complement_edge_ = false;  ///< Flag for complement edge.
   double p_ = 0;  ///< Probability of the function graph.
@@ -426,6 +422,7 @@ class UniqueTable {
   void clear() {
     for (Bucket& chain : table_)
       chain.clear();
+    size_ = 0;
   }
 
   /// Releases all the memory associated with managing this table with BDD.
@@ -438,9 +435,7 @@ class UniqueTable {
   ///       considering the responsibilities of the BDD.
   ///       The release keeps the data about the table,
   ///       such as its size and capacity.
-  void Release() {
-    Table().swap(table_);
-  }
+  void Release() { table_ = Table(); }
 
   /// Finds an existing BDD vertex or
   /// inserts a default constructed weak pointer for a new vertex.
@@ -616,7 +611,7 @@ class CacheTable {
   ///       Using after release of memory is undefined.
   void reserve(int n) {
     if (size_ == 0 && n == 0) {
-      decltype(table_)().swap(table_);
+      table_ = decltype(table_)();
       return;
     }
     if (n <= size_)
@@ -691,7 +686,7 @@ class CacheTable {
 
 class Zbdd;  // For analysis purposes.
 
-/// Analysis of Boolean graphs with Binary Decision Diagrams.
+/// Analysis of PDAGs with Binary Decision Diagrams.
 /// This binary decision diagram data structure
 /// represents Reduced Ordered BDD with attributed edges.
 ///
@@ -708,7 +703,7 @@ class Bdd : private boost::noncopyable {
     VertexPtr vertex;  ///< The root vertex of the BDD function graph.
 
     /// @returns true if the function is initialized.
-    operator bool() const { return vertex != nullptr; }
+    explicit operator bool() const { return vertex != nullptr; }
 
     /// Clears the function's root vertex pointer.
     void reset() { vertex = nullptr; }
@@ -731,23 +726,21 @@ class Bdd : private boost::noncopyable {
     /// @param[in] complement  Interpretation of the BDD vertex.
     ///
     /// @returns The consensus BDD function.
-    static Function Calculate(Bdd* bdd, const ItePtr& ite,
-                              bool complement) noexcept {
+    Function operator()(Bdd* bdd, const ItePtr& ite, bool complement) noexcept {
       return bdd->CalculateConsensus(ite, complement);
     }
   };
 
   /// Constructor with the analysis target.
-  /// Reduced Ordered BDD is produced from a Boolean graph.
+  /// Reduced Ordered BDD is produced from a PDAG.
   ///
-  /// @param[in] fault_tree  Preprocessed, partially normalized,
-  ///                        and indexed fault tree.
+  /// @param[in] graph  Preprocessed and partially normalized PDAG.
   /// @param[in] settings  The analysis settings.
   ///
-  /// @pre The Boolean graph has variable ordering.
+  /// @pre The PDAG has variable ordering.
   ///
   /// @note BDD construction may take considerable time.
-  Bdd(const BooleanGraph* fault_tree, const Settings& settings);
+  Bdd(const Pdag* graph, const Settings& settings);
 
   /// To handle incomplete ZBDD type with unique pointers.
   ~Bdd() noexcept;
@@ -755,7 +748,7 @@ class Bdd : private boost::noncopyable {
   /// @returns The root function of the ROBDD.
   const Function& root() const { return root_; }
 
-  /// @returns Mapping of Boolean graph modules and BDD graph vertices.
+  /// @returns Mapping of PDAG modules and BDD graph vertices.
   const std::unordered_map<int, Function>& modules() const { return modules_; }
 
   /// @returns Mapping of variable indices to their orders.
@@ -777,13 +770,16 @@ class Bdd : private boost::noncopyable {
   void ClearMarks(bool mark) { ClearMarks(root_.vertex, mark); }
 
   /// Runs the Qualitative analysis
-  /// with the representation of a Boolean graph as ROBDD.
+  /// with the representation of a PDAG as ROBDD.
   void Analyze() noexcept;
 
   /// @returns Products generated by the analysis.
   ///
   /// @pre Analysis is done.
-  const std::vector<std::vector<int>>& products() const;
+  const Zbdd& products() const {
+    assert(zbdd_ && "Analysis is not done.");
+    return *zbdd_;
+  }
 
  private:
   using IteWeakPtr = WeakIntrusivePtr<Ite>;  ///< Pointer in containers.
@@ -835,7 +831,7 @@ class Bdd : private boost::noncopyable {
   ItePtr FindOrAddVertex(const Gate& gate, const VertexPtr& high,
                          const VertexPtr& low, bool complement_edge) noexcept;
 
-  /// Converts all gates in the Boolean graph
+  /// Converts all gates in the PDAG
   /// into function BDD graphs.
   /// Registers processed gates.
   ///
@@ -956,6 +952,17 @@ class Bdd : private boost::noncopyable {
     or_table_.clear();
   }
 
+  /// Freezes the graph.
+  /// Releases all possible memory from memoization and unique tables.
+  ///
+  /// @pre No more graph modifications after the freeze.
+  void Freeze() noexcept {
+    unique_table_.Release();
+    ClearTables();
+    and_table_.reserve(0);
+    or_table_.reserve(0);
+  }
+
   const Settings kSettings_;  ///< Analysis settings.
   Function root_;  ///< The root function of this BDD.
   bool coherent_;  ///< Inherited coherence from PDAG.
