@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Olzhas Rakhimov
+ * Copyright (C) 2014-2017 Olzhas Rakhimov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,15 +28,18 @@ namespace scram {
 /// Provides environmental variables.
 class Env {
  public:
-  /// @returns The location of the RelaxNG schema for configuration files.
+  /// @returns The location of the RELAX NG schema for configuration files.
   static std::string config_schema();
 
-  /// @returns The location of the RelaxNG schema for input files.
+  /// @returns The location of the RELAX NG schema for input files.
   static std::string input_schema();
 
-  /// @returns The location of the RelaxNG schema for output report files.
+  /// @returns The location of the RELAX NG schema for output report files.
   static std::string report_schema();
 
+  /// @returns The path to the installation directory.
+  static const std::string& install_dir() { return kInstallDir_; }
+
  private:
   static const std::string kInstallDir_;  ///< Installation directory.
 };
