@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Olzhas Rakhimov
+ * Copyright (C) 2014-2017 Olzhas Rakhimov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,12 +20,12 @@
 
 #include "expression.h"
 
-#include "ext.h"
+#include "ext/algorithm.h"
 
 namespace scram {
 namespace mef {
 
-Expression::Expression(std::vector<ExpressionPtr> args)
+Expression::Expression(std::vector<Expression*> args)
     : args_(std::move(args)),
       sampled_value_(0),
       sampled_(false) {}
@@ -33,7 +33,7 @@ Expression::Expression(std::vector<ExpressionPtr> args)
 double Expression::Sample() noexcept {
   if (!sampled_) {
     sampled_ = true;
-    sampled_value_ = this->GetSample();
+    sampled_value_ = this->DoSample();
   }
   return sampled_value_;
 }
@@ -42,13 +42,12 @@ void Expression::Reset() noexcept {
   if (!sampled_)
     return;
   sampled_ = false;
-  for (const ExpressionPtr& arg : args_)
+  for (Expression* arg : args_)
     arg->Reset();
 }
 
-bool Expression::IsConstant() noexcept {
-  return ext::all_of(
-      args_, [](const ExpressionPtr& arg) { return arg->IsConstant(); });
+bool Expression::IsDeviate() noexcept {
+  return ext::any_of(args_, [](Expression* arg) { return arg->IsDeviate(); });
 }
 
 }  // namespace mef
