@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Olzhas Rakhimov
+ * Copyright (C) 2014-2017 Olzhas Rakhimov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,32 +22,13 @@
 
 #include <boost/math/constants/constants.hpp>
 
-#include "src/error.h"
-
 namespace scram {
 namespace mef {
 
-const ExpressionPtr ConstantExpression::kOne(new ConstantExpression(1));
-const ExpressionPtr ConstantExpression::kZero(new ConstantExpression(0));
-const ExpressionPtr ConstantExpression::kPi(
-    new ConstantExpression(boost::math::constants::pi<double>()));
-
-ConstantExpression::ConstantExpression(double value)
-    : Expression({}),
-      value_(value) {}
-
-ConstantExpression::ConstantExpression(int value)
-    : ConstantExpression(static_cast<double>(value)) {}
-
-ConstantExpression::ConstantExpression(bool value)
-    : ConstantExpression(static_cast<double>(value)) {}
-
-MissionTime::MissionTime(double time, Units unit)
-    : ConstantExpression(time),
-      unit_(unit) {
-  if (time < 0)
-    throw LogicError("Mission time cannot be negative.");
-}
+ConstantExpression ConstantExpression::kOne(1);
+ConstantExpression ConstantExpression::kZero(0);
+ConstantExpression ConstantExpression::kPi(
+    boost::math::constants::pi<double>());
 
 }  // namespace mef
 }  // namespace scram
