@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Olzhas Rakhimov
+ * Copyright (C) 2014-2017 Olzhas Rakhimov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,20 +25,22 @@
 namespace scram {
 namespace mef {
 
-Parameter::Parameter(std::string name, std::string base_path,
-                     RoleSpecifier role)
-    : Expression({}),
-      Element(std::move(name)),
-      Role(role, std::move(base_path)),
-      Id(*this, *this),
-      unit_(kUnitless),
-      unused_(true),
-      expression_(nullptr) {}
+MissionTime::MissionTime(double time, Units unit)
+    : unit_(unit),
+      value_(time) {
+  value(time);
+}
 
-void Parameter::expression(const ExpressionPtr& expression) {
+void MissionTime::value(double time) {
+  if (time < 0)
+    throw LogicError("Mission time cannot be negative.");
+  value_ = time;
+}
+
+void Parameter::expression(Expression* expression) {
   if (expression_)
     throw LogicError("Parameter expression is already set.");
-  expression_ = expression.get();
+  expression_ = expression;
   Expression::AddArg(expression);
 }
 
