@@ -114,11 +114,11 @@ class Reporter {
   /// Reports the results of event tree analysis
   /// to a specified output destination.
   ///
-  /// @param[in] eta  Event Tree Analysis with sequence results.
+  /// @param[in] eta_result  Event Tree Analysis with sequence results.
   /// @param[in,out] results  XML element to for all results.
   ///
   /// @pre The probability analysis has been performed.
-  void ReportResults(const core::EventTreeAnalysis& eta,
+  void ReportResults(const core::RiskAnalysis::EtaResult& eta_result,
                      XmlStreamElement* results);
 
   /// Reports the results of fault tree analysis
