@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Olzhas Rakhimov
+ * Copyright (C) 2014-2017 Olzhas Rakhimov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,8 +37,11 @@ const char* core();
 /// @returns The version of the boost.
 const char* boost();
 
-/// @returns The version of XML libraries.
-const char* xml2();
+/// @returns The version of the XML C++ wrapper.
+const char* libxmlpp();
+
+/// @returns The version of the XML C library.
+const char* libxml();
 
 }  // namespace version
 }  // namespace scram
