@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2016 Olzhas Rakhimov
+ * Copyright (C) 2015-2017 Olzhas Rakhimov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,11 +20,14 @@
 
 #include "zbdd.h"
 
+#include <cstdlib>
+
 #include <algorithm>
 
 #include <boost/range/algorithm.hpp>
 
-#include "ext.h"
+#include "ext/algorithm.h"
+#include "ext/find_iterator.h"
 #include "logger.h"
 
 namespace scram {
@@ -62,53 +65,38 @@ Zbdd::Zbdd(Bdd* bdd, const Settings& settings) noexcept
   CHECK_ZBDD(true);
 }
 
-Zbdd::Zbdd(const BooleanGraph* fault_tree, const Settings& settings) noexcept
-    : Zbdd(*fault_tree->root(), settings) {
-  assert(!fault_tree->complement() && "Complements must be propagated.");
-  if (fault_tree->root()->IsConstant()) {
-    if (fault_tree->root()->state() == kNullState) {
-      root_ = kEmpty_;
-    } else {
-      root_ = kBase_;
-    }
-  } else if (fault_tree->root()->type() == kNull) {
-    const GatePtr& top_gate = fault_tree->root();
-    assert(top_gate->args().size() == 1);
-    assert(top_gate->args<Gate>().empty());
-    int child = *top_gate->args().begin();
-    if (child < 0) {
+Zbdd::Zbdd(const Pdag* graph, const Settings& settings) noexcept
+    : Zbdd(graph->root(), settings) {
+  assert(!graph->complement() && "Complements must be propagated.");
+  if (graph->IsTrivial()) {
+    const Gate& top_gate = graph->root();
+    assert(top_gate.args().size() == 1);
+    assert(top_gate.args<Gate>().empty());
+    int child = *top_gate.args().begin();
+    if (top_gate.constant()) {
+      root_ = child < 0 ? kEmpty_ : kBase_;
+    } else if (child < 0) {
       root_ = kBase_;
     } else {
-      const VariablePtr& var = top_gate->args<Variable>().begin()->second;
-      root_ = FindOrAddVertex(var->index(), kBase_, kEmpty_, var->order());
+      const Variable& var = top_gate.args<Variable>().begin()->second;
+      root_ = FindOrAddVertex(var.index(), kBase_, kEmpty_, var.order());
     }
   }
   CHECK_ZBDD(true);
 }
 
 void Zbdd::Analyze() noexcept {
+  CLOCK(zbdd_time);
   assert(root_->terminal() ||
-         SetNode::Ptr(root_)->max_set_order() <= kSettings_.limit_order());
+         SetNode::Ref(root_).max_set_order() <= kSettings_.limit_order());
   root_ = Minimize(root_);  // Likely to be minimal by now.
-  assert(root_->terminal() || SetNode::Ptr(root_)->minimal());
+  assert(root_->terminal() || SetNode::Ref(root_).minimal());
   for (const auto& entry : modules_)
     entry.second->Analyze();
 
-  CLOCK(gen_time);
-  LOG(DEBUG3) << "Getting products from minimized ZBDD: G" << module_index_;
-  // Complete cleanup of the memory.
-  ReleaseTables();
-  ClearMarks(root_, false);
-  ClearCounts(root_, false);
-  EncodeLimitOrder(root_, kSettings_.limit_order());
-  ClearMarks(root_, false);
-  products_ = GenerateProducts(root_);
-
-  // Cleanup of temporary products.
-  modules_.clear();
-  root_ = kEmpty_;
-  LOG(DEBUG4) << "# of generated products: " << products_.size();
-  LOG(DEBUG3) << "G" << module_index_ << " analysis time: " << DUR(gen_time);
+  Prune(root_, kSettings_.limit_order());
+  Freeze();  // Complete cleanup of the memory.
+  LOG(DEBUG3) << "G" << module_index_ << " analysis time: " << DUR(zbdd_time);
 }
 
 Zbdd::Zbdd(const Settings& settings, bool coherent, int module_index) noexcept
@@ -129,7 +117,7 @@ Zbdd::Zbdd(const Bdd::Function& module, bool coherent, Bdd* bdd,
   PairTable<VertexPtr> ites;
   root_ = Minimize(ConvertBdd(module.vertex, module.complement, bdd,
                               kSettings_.limit_order(), &ites));
-  assert(root_->terminal() || SetNode::Ptr(root_)->minimal());
+  assert(root_->terminal() || SetNode::Ref(root_).minimal());
   Log();
   LOG(DEBUG2) << "Created ZBDD from BDD in " << DUR(init_time);
   std::map<int, std::pair<bool, int>> sub_modules;
@@ -163,7 +151,7 @@ Zbdd::Zbdd(const Bdd::Function& module, bool coherent, Bdd* bdd,
 
 Zbdd::Zbdd(const Gate& gate, const Settings& settings) noexcept
     : Zbdd(settings, gate.coherent(), gate.index()) {
-  if (gate.IsConstant() || gate.type() == kNull)
+  if (gate.constant() || gate.type() == kNull)
     return;
   assert(!settings.prime_implicants() && "Not implemented.");
   CLOCK(init_time);
@@ -218,9 +206,9 @@ SetNodePtr Zbdd::FindOrAddVertex(int index, const VertexPtr& high,
   SetNodePtr node(new SetNode(index, order, set_id_++, high, low));
   node->module(module);
   node->coherent(coherent);
-  int high_order = high->terminal() ? 0 : SetNode::Ptr(high)->max_set_order();
-  high_order += !MayBeUnity(node);
-  int low_order = low->terminal() ? 0 : SetNode::Ptr(low)->max_set_order();
+  int high_order = high->terminal() ? 0 : SetNode::Ref(high).max_set_order();
+  high_order += !MayBeUnity(*node);
+  int low_order = low->terminal() ? 0 : SetNode::Ref(low).max_set_order();
   node->max_set_order(std::max(high_order, low_order));
 
   in_table = node;
@@ -246,9 +234,9 @@ Zbdd::VertexPtr Zbdd::GetReducedVertex(const ItePtr& ite, bool complement,
                                        const VertexPtr& low) noexcept {
   if (high->id() == low->id())
     return low;
-  if (high->terminal() && !Terminal<SetNode>::Ptr(high)->value())
+  if (high->terminal() && !Terminal<SetNode>::Ref(high).value())
     return low;
-  if (low->terminal() && Terminal<SetNode>::Ptr(low)->value())
+  if (low->terminal() && Terminal<SetNode>::Ref(low).value())
     return low;
   assert(ite->index() > 0 && "BDD indices are never negative.");
   return FindOrAddVertex(complement ? -ite->index() : ite->index(),
@@ -261,9 +249,9 @@ Zbdd::VertexPtr Zbdd::GetReducedVertex(const SetNodePtr& node,
                                        const VertexPtr& low) noexcept {
   if (high->id() == low->id())
     return low;
-  if (high->terminal() && !Terminal<SetNode>::Ptr(high)->value())
+  if (high->terminal() && !Terminal<SetNode>::Ref(high).value())
     return low;
-  if (low->terminal() && Terminal<SetNode>::Ptr(low)->value())
+  if (low->terminal() && Terminal<SetNode>::Ref(low).value())
     return low;
   if (node->high()->id() == high->id() && node->low()->id() == low->id())
     return node;
@@ -287,7 +275,7 @@ Zbdd::VertexPtr Zbdd::ConvertBdd(const Bdd::VertexPtr& vertex, bool complement,
         ConvertBdd(Ite::Ptr(vertex), complement, bdd_graph, limit_order, ites);
   }
   assert(result->terminal() ||
-         SetNode::Ptr(result)->max_set_order() <= limit_order);
+         SetNode::Ref(result).max_set_order() <= limit_order);
   return result;
 }
 
@@ -313,7 +301,7 @@ Zbdd::VertexPtr
 Zbdd::ConvertBddPrimeImplicants(const ItePtr& ite, bool complement,
                                 Bdd* bdd_graph, int limit_order,
                                 PairTable<VertexPtr>* ites) noexcept {
-  Bdd::Function common = Bdd::Consensus::Calculate(bdd_graph, ite, complement);
+  Bdd::Function common = Bdd::Consensus()(bdd_graph, ite, complement);
   VertexPtr consensus = ConvertBdd(common.vertex, common.complement, bdd_graph,
                                    limit_order, ites);
   if (limit_order == 0) {  // Cut-off on the product order.
@@ -338,7 +326,7 @@ Zbdd::VertexPtr Zbdd::ConvertGraph(
     const Gate& gate,
     std::unordered_map<int, std::pair<VertexPtr, int>>* gates,
     std::unordered_map<int, const Gate*>* module_gates) noexcept {
-  assert(!gate.IsConstant() && "Unexpected constant gate!");
+  assert(!gate.constant() && "Unexpected constant gate!");
   VertexPtr result;
   if (auto it_entry = ext::find(*gates, gate.index())) {
     std::pair<VertexPtr, int>& entry = it_entry->second;
@@ -349,17 +337,17 @@ Zbdd::VertexPtr Zbdd::ConvertGraph(
     return result;
   }
   std::vector<VertexPtr> args;
-  for (const Gate::Arg<Variable>& arg : gate.args<Variable>()) {
+  for (const Gate::ConstArg<Variable>& arg : gate.args<Variable>()) {
     args.push_back(
-        FindOrAddVertex(arg.first, kBase_, kEmpty_, arg.second->order()));
+        FindOrAddVertex(arg.first, kBase_, kEmpty_, arg.second.order()));
   }
-  for (const Gate::Arg<Gate>& arg : gate.args<Gate>()) {
+  for (const Gate::ConstArg<Gate>& arg : gate.args<Gate>()) {
     assert(arg.first > 0 && "Complements must be pushed down to variables.");
-    if (arg.second->module()) {
-      module_gates->emplace(arg.first, arg.second.get());
-      args.push_back(FindOrAddVertex(*arg.second, kBase_, kEmpty_));
+    if (arg.second.module()) {
+      module_gates->emplace(arg.first, &arg.second);
+      args.push_back(FindOrAddVertex(arg.second, kBase_, kEmpty_));
     } else {
-      args.push_back(ConvertGraph(*arg.second, gates, module_gates));
+      args.push_back(ConvertGraph(arg.second, gates, module_gates));
     }
   }
   boost::sort(args, [](const VertexPtr& lhs, const VertexPtr& rhs) {
@@ -367,7 +355,7 @@ Zbdd::VertexPtr Zbdd::ConvertGraph(
       return true;
     if (rhs->terminal())
       return false;
-    return SetNode::Ptr(lhs)->order() > SetNode::Ptr(rhs)->order();
+    return SetNode::Ref(lhs).order() > SetNode::Ref(rhs).order();
   });
   auto it = args.cbegin();
   for (result = *it++; it != args.cend(); ++it) {
@@ -376,7 +364,7 @@ Zbdd::VertexPtr Zbdd::ConvertGraph(
   ClearTables();
   assert(result);
   assert(result->terminal() ||
-         SetNode::Ptr(result)->max_set_order() <= kSettings_.limit_order());
+         SetNode::Ref(result).max_set_order() <= kSettings_.limit_order());
   if (gate.parents().size() > 1)
     gates->insert({gate.index(), {result, 1}});
   return result;
@@ -412,7 +400,7 @@ Zbdd::VertexPtr Zbdd::Apply<kAnd>(const SetNodePtr& arg_one,
                                   int limit_order) noexcept {
   VertexPtr high;
   VertexPtr low;
-  int limit_high = limit_order - !MayBeUnity(arg_one);
+  int limit_high = limit_order - !MayBeUnity(*arg_one);
   if (arg_one->order() == arg_two->order() &&
       arg_one->index() == arg_two->index()) {  // The same variable.
     // (x*f1 + f0) * (x*g1 + g0) = x*(f1*(g1 + g0) + f0*g1) + f0*g0
@@ -434,9 +422,9 @@ Zbdd::VertexPtr Zbdd::Apply<kAnd>(const SetNodePtr& arg_one,
     }
     low = Apply<kAnd>(arg_one->low(), arg_two, limit_order);
   }
-  if (!high->terminal() && SetNode::Ptr(high)->order() == arg_one->order()) {
-    assert(SetNode::Ptr(high)->index() < arg_one->index());
-    high = SetNode::Ptr(high)->low();
+  if (!high->terminal() && SetNode::Ref(high).order() == arg_one->order()) {
+    assert(SetNode::Ref(high).index() < arg_one->index());
+    high = SetNode::Ref(high).low();
   }
   return Minimize(GetReducedVertex(arg_one, high, low));
 }
@@ -449,12 +437,12 @@ Zbdd::VertexPtr Zbdd::Apply<kAnd>(const VertexPtr& arg_one,
   if (limit_order < 0)
     return kEmpty_;
   if (arg_one->terminal()) {
-    if (Terminal<SetNode>::Ptr(arg_one)->value())
+    if (Terminal<SetNode>::Ref(arg_one).value())
       return Prune(arg_two, limit_order);
     return kEmpty_;
   }
   if (arg_two->terminal()) {
-    if (Terminal<SetNode>::Ptr(arg_two)->value())
+    if (Terminal<SetNode>::Ref(arg_two).value())
       return Prune(arg_one, limit_order);
     return kEmpty_;
   }
@@ -476,7 +464,7 @@ Zbdd::VertexPtr Zbdd::Apply<kAnd>(const VertexPtr& arg_one,
   }
   result = Apply<kAnd>(set_one, set_two, limit_order);
   assert(result->terminal() ||
-         SetNode::Ptr(result)->max_set_order() <= limit_order);
+         SetNode::Ref(result).max_set_order() <= limit_order);
   return result;
 }
 
@@ -487,7 +475,7 @@ Zbdd::VertexPtr Zbdd::Apply<kOr>(const SetNodePtr& arg_one,
                                  int limit_order) noexcept {
   VertexPtr high;
   VertexPtr low;
-  int limit_high = limit_order - !MayBeUnity(arg_one);
+  int limit_high = limit_order - !MayBeUnity(*arg_one);
   if (arg_one->order() == arg_two->order() &&
       arg_one->index() == arg_two->index()) {  // The same variable.
     high = Apply<kOr>(arg_one->high(), arg_two->high(), limit_high);
@@ -503,9 +491,9 @@ Zbdd::VertexPtr Zbdd::Apply<kOr>(const SetNodePtr& arg_one,
     high = Prune(arg_one->high(), limit_high);
     low = Apply<kOr>(arg_one->low(), arg_two, limit_order);
   }
-  if (!high->terminal() && SetNode::Ptr(high)->order() == arg_one->order()) {
-    assert(SetNode::Ptr(high)->index() < arg_one->index());
-    high = SetNode::Ptr(high)->low();
+  if (!high->terminal() && SetNode::Ref(high).order() == arg_one->order()) {
+    assert(SetNode::Ref(high).index() < arg_one->index());
+    high = SetNode::Ref(high).low();
   }
   return Minimize(GetReducedVertex(arg_one, high, low));
 }
@@ -518,12 +506,12 @@ Zbdd::VertexPtr Zbdd::Apply<kOr>(const VertexPtr& arg_one,
   if (limit_order < 0)
     return kEmpty_;
   if (arg_one->terminal()) {
-    if (Terminal<SetNode>::Ptr(arg_one)->value())
+    if (Terminal<SetNode>::Ref(arg_one).value())
       return kBase_;
     return Prune(arg_two, limit_order);
   }
   if (arg_two->terminal()) {
-    if (Terminal<SetNode>::Ptr(arg_two)->value())
+    if (Terminal<SetNode>::Ref(arg_two).value())
       return kBase_;
     return Prune(arg_one, limit_order);
   }
@@ -545,7 +533,7 @@ Zbdd::VertexPtr Zbdd::Apply<kOr>(const VertexPtr& arg_one,
   }
   result = Apply<kOr>(set_one, set_two, limit_order);
   assert(result->terminal() ||
-         SetNode::Ptr(result)->max_set_order() <= limit_order);
+         SetNode::Ref(result).max_set_order() <= limit_order);
   return result;
 }
 
@@ -579,9 +567,9 @@ Zbdd::VertexPtr Zbdd::EliminateComplement(const SetNodePtr& node,
                                           const VertexPtr& low) noexcept {
   // Cut-off does not matter with the OR operation with conforming inputs.
   assert(high->terminal() ||
-         SetNode::Ptr(high)->terminal() <= kSettings_.limit_order());
+         SetNode::Ref(high).max_set_order() <= kSettings_.limit_order());
   assert(low->terminal() ||
-         SetNode::Ptr(low)->terminal() <= kSettings_.limit_order());
+         SetNode::Ref(low).max_set_order() <= kSettings_.limit_order());
   if (node->index() < 0 && !(node->module() && !node->coherent()))
     return Apply<kOr>(high, low, kSettings_.limit_order());
   return Minimize(GetReducedVertex(node, high, low));
@@ -619,7 +607,7 @@ Zbdd::VertexPtr Zbdd::EliminateConstantModule(const SetNodePtr& node,
   if (node->module()) {
     Zbdd* module = modules_.find(node->index())->second.get();
     if (module->root_->terminal()) {
-      if (!Terminal<SetNode>::Ptr(module->root_)->value())
+      if (!Terminal<SetNode>::Ref(module->root_).value())
         return low;
       return Apply<kOr>(high, low, kSettings_.limit_order());
     }
@@ -640,19 +628,19 @@ Zbdd::VertexPtr Zbdd::Minimize(const VertexPtr& vertex) noexcept {
   VertexPtr low = Minimize(node->low());
   high = Subsume(high, low);
   assert(high->id() != low->id() && "Subsume failed!");
-  if (high->terminal() && !Terminal<SetNode>::Ptr(high)->value()) {
+  if (high->terminal() && !Terminal<SetNode>::Ref(high).value()) {
     result = low;  // Reduction rule.
     return result;
   }
   result = FindOrAddVertex(node, high, low);
-  SetNode::Ptr(result)->minimal(true);
+  SetNode::Ref(result).minimal(true);
   return result;
 }
 
 Zbdd::VertexPtr Zbdd::Subsume(const VertexPtr& high,
                               const VertexPtr& low) noexcept {
   if (low->terminal())
-    return Terminal<SetNode>::Ptr(low)->value() ? kEmpty_ : high;
+    return Terminal<SetNode>::Ref(low).value() ? kEmpty_ : high;
   if (high->terminal())
     return high;  // No need to reduce terminal sets.
   VertexPtr& computed = subsume_table_[{high->id(), low->id()}];
@@ -682,7 +670,7 @@ Zbdd::VertexPtr Zbdd::Subsume(const VertexPtr& high,
     subhigh = Subsume(high_node->high(), low);
     sublow = Subsume(high_node->low(), low);
   }
-  if (subhigh->terminal() && !Terminal<SetNode>::Ptr(subhigh)->value()) {
+  if (subhigh->terminal() && !Terminal<SetNode>::Ref(subhigh).value()) {
     computed = sublow;
     return computed;
   }
@@ -707,26 +695,26 @@ Zbdd::VertexPtr Zbdd::Prune(const VertexPtr& vertex, int limit_order) noexcept {
   if (result)
     return result;
 
-  int limit_high = limit_order - !MayBeUnity(node);
+  int limit_high = limit_order - !MayBeUnity(*node);
   result = GetReducedVertex(node, Prune(node->high(), limit_high),
                             Prune(node->low(), limit_order));
   if (!result->terminal())
-    SetNode::Ptr(result)->minimal(node->minimal());
+    SetNode::Ref(result).minimal(node->minimal());
   return result;
 }
 
-bool Zbdd::MayBeUnity(const SetNodePtr& node) noexcept {
+bool Zbdd::MayBeUnity(const SetNode& node) noexcept {
   if (kSettings_.prime_implicants())
     return false;
   // Unity node tests for minimal cut sets.
-  if (node->index() < 0)
+  if (node.index() < 0)
     return true;  // Unity complement vars.
   // Non-modular gates can be implied by other gates in the product;
   // that is, (G1 & G2 = I & G2) if G2 implies G1.
   //
   // Non-coherent gates contain complements to be approximated to Unity.
   if (this->IsGate(node))
-    return !node->module() || !node->coherent();
+    return !node.module() || !node.coherent();
   return false;  // Positive non-gate variable.
 }
 
@@ -735,190 +723,127 @@ int Zbdd::GatherModules(const VertexPtr& vertex,
                         std::map<int, std::pair<bool, int>>* modules) noexcept {
   assert(current_order >= 0);
   if (vertex->terminal())
-    return Terminal<SetNode>::Ptr(vertex)->value() ? 0 : -1;
-  SetNodePtr node = SetNode::Ptr(vertex);
+    return Terminal<SetNode>::Ref(vertex).value() ? 0 : -1;
+  SetNode& node = SetNode::Ref(vertex);
   int contribution = !MayBeUnity(node);
   int high_order = current_order + contribution;
-  int min_high = GatherModules(node->high(), high_order, modules);
+  int min_high = GatherModules(node.high(), high_order, modules);
   assert(min_high >= 0 && "No terminal Empty should be on high branch.");
-  if (node->module()) {
+  if (node.module()) {
     int module_order = kSettings_.limit_order() - min_high - current_order;
     assert(module_order >= 0 && "Improper application of a cut-off.");
-    if (auto it = ext::find(*modules, node->index())) {
+    if (auto it = ext::find(*modules, node.index())) {
       std::pair<bool, int>& entry = it->second;
-      assert(entry.first == node->coherent() && "Inconsistent flags.");
+      assert(entry.first == node.coherent() && "Inconsistent flags.");
       entry.second = std::max(entry.second, module_order);
     } else {
-      modules->insert({node->index(), {node->coherent(), module_order}});
+      modules->insert({node.index(), {node.coherent(), module_order}});
     }
   }
-  int min_low = GatherModules(node->low(), current_order, modules);
+  int min_low = GatherModules(node.low(), current_order, modules);
   assert(min_low >= -1);
   if (min_low == -1)
     return min_high + contribution;
   return std::min(min_high + contribution, min_low);
 }
 
-void Zbdd::EncodeLimitOrder(const VertexPtr& vertex, int limit_order) noexcept {
-  if (vertex->terminal())
-    return;
-  SetNodePtr node = SetNode::Ptr(vertex);
-  if (node->count() >= limit_order)
-    return;
-  node->count(limit_order);
-  EncodeLimitOrder(node->high(), limit_order - 1);
-  EncodeLimitOrder(node->low(), limit_order);
-}
-
-std::vector<std::vector<int>>
-Zbdd::GenerateProducts(const VertexPtr& vertex) noexcept {
-  if (vertex->terminal()) {
-    if (Terminal<SetNode>::Ptr(vertex)->value())
-      return {{}};  // The Base set.
-    return {};  // Don't include 0/NULL sets.
-  }
-  SetNodePtr node = SetNode::Ptr(vertex);
-  assert(node->minimal() && "Detected non-minimal ZBDD.");
-  if (node->count() <= 0)
-    return {};  // The result of a conservative count.
-
-  if (node->mark())
-    return node->products();
-  node->mark(true);
-  std::vector<Product> low = GenerateProducts(node->low());
-  std::vector<Product> high = GenerateProducts(node->high());
-  std::vector<Product> result;
-  for (auto& product : low) {
-    if (product.size() <= node->count())
-      result.emplace_back(std::move(product));
-  }
-  if (node->module()) {
-    Zbdd* module = modules_.find(node->index())->second.get();
-    for (const auto& product : high) {  // Cross-product.
-      for (const auto& module_set : module->products()) {
-        if (product.size() + module_set.size() > node->count())
-          continue;  // Cut-off on the product size.
-        Product combo = product;
-        combo.insert(combo.end(), module_set.begin(), module_set.end());
-        result.emplace_back(std::move(combo));
-      }
-    }
-  } else {
-    for (auto& product : high) {
-      if (product.size() < node->count()) {
-        product.push_back(node->index());
-        result.emplace_back(std::move(product));
-      }
-    }
-  }
-
-  // Destroy the subgraph to remove extra reference counts.
-  node->CutBranches();
-
-  if (node->use_count() > 2)
-    node->products(result);
-  return result;
-}
-
 int Zbdd::CountSetNodes(const VertexPtr& vertex) noexcept {
   if (vertex->terminal())
     return 0;
-  SetNodePtr node = SetNode::Ptr(vertex);
-  if (node->mark())
+  SetNode& node = SetNode::Ref(vertex);
+  if (node.mark())
     return 0;
-  node->mark(true);
-  return 1 + CountSetNodes(node->high()) + CountSetNodes(node->low());
+  node.mark(true);
+  return 1 + CountSetNodes(node.high()) + CountSetNodes(node.low());
 }
 
 std::int64_t Zbdd::CountProducts(const VertexPtr& vertex,
                                  bool modules) noexcept {
-  if (vertex->terminal()) {
-    if (Terminal<SetNode>::Ptr(vertex)->value())
-      return 1;
-    return 0;
-  }
-  SetNodePtr node = SetNode::Ptr(vertex);
-  if (node->mark())
-    return node->count();
-  node->mark(true);
+  if (vertex->terminal())
+    return Terminal<SetNode>::Ref(vertex).value();
+
+  SetNode& node = SetNode::Ref(vertex);
+  if (node.mark())
+    return node.count();
+  node.mark(true);
   std::int64_t multiplier = 1;  // Multiplier of the module.
-  if (modules && node->module()) {
-    Zbdd* module = modules_.find(node->index())->second.get();
+  if (modules && node.module()) {
+    Zbdd* module = modules_.find(node.index())->second.get();
     multiplier = module->CountProducts(module->root_, true);
   }
-  node->count(multiplier * CountProducts(node->high(), modules) +
-              CountProducts(node->low(), modules));
-  return node->count();
+  node.count(multiplier * CountProducts(node.high(), modules) +
+             CountProducts(node.low(), modules));
+  return node.count();
 }
 
 void Zbdd::ClearMarks(const VertexPtr& vertex, bool modules) noexcept {
   if (vertex->terminal())
     return;
-  SetNodePtr node = SetNode::Ptr(vertex);
-  if (!node->mark())
+  SetNode& node = SetNode::Ref(vertex);
+  if (!node.mark())
     return;
-  node->mark(false);
-  if (modules && node->module()) {
-    Zbdd* module = modules_.find(node->index())->second.get();
+  node.mark(false);
+  if (modules && node.module()) {
+    Zbdd* module = modules_.find(node.index())->second.get();
     module->ClearMarks(module->root_, true);
   }
-  ClearMarks(node->high(), modules);
-  ClearMarks(node->low(), modules);
+  ClearMarks(node.high(), modules);
+  ClearMarks(node.low(), modules);
 }
 
 void Zbdd::ClearCounts(const VertexPtr& vertex, bool modules) noexcept {
   if (vertex->terminal())
     return;
-  SetNodePtr node = SetNode::Ptr(vertex);
-  if (node->mark())
+  SetNode& node = SetNode::Ref(vertex);
+  if (node.mark())
     return;
-  node->mark(true);
-  node->count(0);
-  if (modules && node->module()) {
-    Zbdd* module = modules_.find(node->index())->second.get();
+  node.mark(true);
+  node.count(0);
+  if (modules && node.module()) {
+    Zbdd* module = modules_.find(node.index())->second.get();
     module->ClearCounts(module->root_, true);
   }
-  ClearCounts(node->high(), modules);
-  ClearCounts(node->low(), modules);
+  ClearCounts(node.high(), modules);
+  ClearCounts(node.low(), modules);
 }
 
 void Zbdd::TestStructure(const VertexPtr& vertex, bool modules) noexcept {
   if (vertex->terminal())
     return;
-  SetNodePtr node = SetNode::Ptr(vertex);
-  if (node->mark())
+  SetNode& node = SetNode::Ref(vertex);
+  if (node.mark())
     return;
-  node->mark(true);
-  assert(node->index() && "Illegal index for a node.");
-  assert(node->order() && "Improper order for nodes.");
-  assert(node->high() && node->low() && "Malformed node high/low pointers.");
-  assert(!(node->high()->terminal() &&
-           !Terminal<SetNode>::Ptr(node->high())->value()) &&
+  node.mark(true);
+  assert(node.index() && "Illegal index for a node.");
+  assert(node.order() && "Improper order for nodes.");
+  assert(node.high() && node.low() && "Malformed node high/low pointers.");
+  assert(!(node.high()->terminal() &&
+           !Terminal<SetNode>::Ref(node.high()).value()) &&
          "Reduction rule failure.");
-  assert((node->high()->id() != node->low()->id()) && "Minimization failure.");
-  assert(!(!node->high()->terminal() &&
-           node->order() >= SetNode::Ptr(node->high())->order()) &&
+  assert((node.high()->id() != node.low()->id()) && "Minimization failure.");
+  assert(!(!node.high()->terminal() &&
+           node.order() >= SetNode::Ref(node.high()).order()) &&
          "Ordering of nodes failed.");
-  assert(!(!node->low()->terminal() &&
-           node->order() > SetNode::Ptr(node->low())->order()) &&
+  assert(!(!node.low()->terminal() &&
+           node.order() > SetNode::Ref(node.low()).order()) &&
          "Ordering of nodes failed.");
-  assert(!(!node->low()->terminal() &&
-           node->order() == SetNode::Ptr(node->low())->order() &&
-           node->index() <= SetNode::Ptr(node->low())->index()) &&
+  assert(!(!node.low()->terminal() &&
+           node.order() == SetNode::Ref(node.low()).order() &&
+           node.index() <= SetNode::Ref(node.low()).index()) &&
          "Ordering of complements failed.");
-  assert(!(!node->high()->terminal() && node->minimal() &&
-           !SetNode::Ptr(node->high())->minimal()) &&
+  assert(!(!node.high()->terminal() && node.minimal() &&
+           !SetNode::Ref(node.high()).minimal()) &&
          "Non-minimal branches in minimal ZBDD.");
-  assert(!(!node->low()->terminal() && node->minimal() &&
-           !SetNode::Ptr(node->low())->minimal()) &&
+  assert(!(!node.low()->terminal() && node.minimal() &&
+           !SetNode::Ref(node.low()).minimal()) &&
          "Non-minimal branches in minimal ZBDD.");
-  if (modules && node->module()) {
-    Zbdd* module = modules_.find(node->index())->second.get();
+  if (modules && node.module()) {
+    Zbdd* module = modules_.find(node.index())->second.get();
     assert(!module->root_->terminal() && "Terminal modules must be removed.");
     module->TestStructure(module->root_, true);
   }
-  TestStructure(node->high(), modules);
-  TestStructure(node->low(), modules);
+  TestStructure(node.high(), modules);
+  TestStructure(node.low(), modules);
 }
 
 namespace zbdd {
@@ -930,16 +855,16 @@ CutSetContainer::CutSetContainer(const Settings& settings, int module_index,
 
 Zbdd::VertexPtr CutSetContainer::ConvertGate(const Gate& gate) noexcept {
   assert(gate.type() == kAnd || gate.type() == kOr);
-  assert(gate.args<Constant>().empty());
+  assert(!gate.constant());
   assert(gate.args().size() > 1);
   std::vector<SetNodePtr> args;
-  for (const Gate::Arg<Variable>& arg : gate.args<Variable>()) {
+  for (const Gate::ConstArg<Variable>& arg : gate.args<Variable>()) {
     args.push_back(
-        FindOrAddVertex(arg.first, kBase_, kEmpty_, arg.second->order()));
+        FindOrAddVertex(arg.first, kBase_, kEmpty_, arg.second.order()));
   }
-  for (const Gate::Arg<Gate>& arg : gate.args<Gate>()) {
+  for (const Gate::ConstArg<Gate>& arg : gate.args<Gate>()) {
     assert(arg.first > 0 && "Complements must be pushed down to variables.");
-    args.push_back(FindOrAddVertex(*arg.second, kBase_, kEmpty_));
+    args.push_back(FindOrAddVertex(arg.second, kBase_, kEmpty_));
   }
   boost::sort(args, [](const SetNodePtr& lhs, const SetNodePtr& rhs) {
     return lhs->order() > rhs->order();
@@ -958,11 +883,11 @@ Zbdd::VertexPtr CutSetContainer::ExtractIntermediateCutSets(
   assert(index && index > gate_index_bound_);
   assert(!root()->terminal() &&
          "Impossible to have intermediate cut sets.");
-  assert(index == SetNode::Ptr(root())->index() && "Broken ordering!");
-  assert(SetNode::Ptr(root())->max_set_order() <= settings().limit_order());
+  assert(index == SetNode::Ref(root()).index() && "Broken ordering!");
+  assert(SetNode::Ref(root()).max_set_order() <= settings().limit_order());
   LOG(DEBUG5) << "Extracting cut sets for G" << index;
   SetNodePtr node = SetNode::Ptr(root());
-  root(node->low());
+  Zbdd::root(node->low());
   return node->high();
 }
 
@@ -970,15 +895,15 @@ Zbdd::VertexPtr
 CutSetContainer::ExpandGate(const VertexPtr& gate_zbdd,
                             const VertexPtr& cut_sets) noexcept {
   assert(gate_zbdd->terminal() ||
-         SetNode::Ptr(gate_zbdd)->max_set_order() <= settings().limit_order());
+         SetNode::Ref(gate_zbdd).max_set_order() <= settings().limit_order());
   assert(cut_sets->terminal() ||
-         SetNode::Ptr(cut_sets)->max_set_order() <= settings().limit_order());
+         SetNode::Ref(cut_sets).max_set_order() <= settings().limit_order());
   return Apply<kAnd>(gate_zbdd, cut_sets, settings().limit_order());
 }
 
 void CutSetContainer::Merge(const VertexPtr& vertex) noexcept {
   assert(vertex->terminal() ||
-         SetNode::Ptr(vertex)->max_set_order() <= settings().limit_order());
+         SetNode::Ref(vertex).max_set_order() <= settings().limit_order());
   root(Apply<kOr>(root(), vertex, settings().limit_order()));
   ClearTables();
 }
