@@ -26,7 +26,7 @@ namespace test {
 TEST_P(RiskAnalysisTest, AttackEventTree) {
   const char* tree_input = "./share/scram/input/EventTrees/attack.xml";
   settings.probability_analysis(true);
-  ASSERT_NO_THROW(ProcessInputFile(tree_input));
+  ASSERT_NO_THROW(ProcessInputFiles({tree_input}));
   ASSERT_NO_THROW(analysis->Analyze());
   EXPECT_EQ(1, analysis->event_tree_results().size());
   const auto& results = sequences();
